
from io import TextIOWrapper
import os
import shutil
import click
from .generate import main as generate_main


@click.group()
def cli():  # pragma: no cover
    pass


@click.command()
@click.argument("file", type=click.File("r"))
@click.option("-O", "--outdir", default="out", help="The directory where the generated CSVs will be stored", type=click.Path(file_okay=False, writable=True), show_default=True)
def generate(file: TextIOWrapper, outdir: str):
    # check if the 'out'-directory already exists
    if os.path.isdir(outdir):
        click.confirm(f"The output directory '{outdir}' and ALL ITS CONTENTS will be REMOVED! Continue?",
                      default=False, abort=True, prompt_suffix=": ", show_default=True, err=False)
        # delete the existing directory
        shutil.rmtree(outdir)

    # create the directory
    try:
        os.mkdir(outdir)
    except OSError as e:
        raise click.UsageError(e)

    generate_main(file, outdir)


cli.add_command(generate)

if __name__ == '__main__':  # pragma: no cover
    cli()



import re
from typing import Dict
from schema import Schema


class ReferenceFieldGenerator():
    option_schema = Schema({
        "field": str,
        "field_attr": str,
    })

    def __init__(self, options: Dict[str, str]):
        self.options = self.option_schema.validate(options)


# e.g. $order.date
reference_regexp = re.compile("^\$(.+)\.(.+)$")


def to_ref(raw: str):
    match = reference_regexp.match(raw)

    if match is None:
        raise Exception(f"Value '{raw}' is not a valid reference!")

    return ReferenceFieldGenerator({
        "field": match[1],
        "field_attr": match[2]
    })



from schema import Or, Schema, Use, Optional, And
from random import random

from .reference import to_ref
from .base import FieldGenerator


class NumberFieldGenerator(FieldGenerator):
    option_schema = Schema({
        "min": Or(Use(to_ref), Use(float)),
        "max": Or(Use(to_ref), Use(float)),
        Optional("digits", default=0): And(Use(int), lambda n: n >= 0)
    })

    def validate(self):
        self.options = self.option_schema.validate(self.options)

    def generate(self, **kwargs):
        options = self.prepare_options(**kwargs)
        min: float = options["min"]
        max: float = options["max"]

        if max < min:
            raise Exception(
                f"'max' ({options['max']}) must be equal or larger than 'min' ({self.options['min']})")

        digits: int = options["digits"]

        return float(f"{((random() * (max - min)) + min):.{digits}f}")

    def val_to_str(self, value) -> str:
        digits: int = self.options["digits"]

        return f"{value:.{digits}f}"



from random import randint
from typing import Dict
from schema import Schema

from .base import FieldGenerator


class SchemaFieldGenerator(FieldGenerator):
    option_schema = Schema({})

    def validate(self):
        self.option_schema.validate(self.options)

    def generate(self, schema_name: str = None, res: Dict = {}, iter_res: Dict = {}):
        schema = iter_res.get(self.type)
        if schema is not None:
            # choose latest
            index = -1
        else:
            # choose random
            schema = res.get(self.type)
            if schema is not None:
                index = randint(0, len(schema[1]) - 1)

        if schema is None:
            raise Exception(f"Schema '{self.type}' is not generated yet")

        self.schema = {}
        for i in range(len(schema[0])):
            self.schema[schema[0][i][0]] = schema[1][index][i]
        return schema[1][index][0]


def get_value_by_header(field_attr: str, schema_name: str, res: Dict = {}, iter_res={}):
    schema = iter_res.get(schema_name, res.get(schema_name))
    if schema is None:
        raise Exception(f"Cannot find schema: 'schema_name' in current result")

    entry = list(filter(
        lambda header: header[1][0] == field_attr, enumerate(schema[0])))[0]

    # check if the entry is a schema field generator
    generator = getattr(entry[1][1], "__self__", None)
    if generator is not None and isinstance(generator, SchemaFieldGenerator):
        # find the referenced schema
        return generator.schema

    return schema[1][-1][entry[0]]
